        MEMBER()
        
        INCLUDE('DirectPrint.INC'),ONCE

DP_PRINTER_DEFAULTS     GROUP,TYPE
pDatatype                   LONG   !*PSTR
pDevMode                    LONG   !LPDEVMODE
DesiredAccess               DP_DWORD
                        END

DP_DOC_INFO_1           GROUP,TYPE
pDocName                    ULONG
pOutputFile                 ULONG
pDataType                   ULONG
                        END

        ! The Following from Clarion's PRNPROP.CLW                        
DP_PROPPRINT:driver          EQUATE(07B27H)  ! driver DLL filename (no extension)
DP_PROPPRINT:device          EQUATE(07B28H)  ! driver description
DP_PROPPRINT:port            EQUATE(07B29H)  ! output port name
DP_PROPPRINT:devmode         EQUATE(07B00H)  ! Entire devmode structure - see WINDOWS SDK

        
        MAP
            MODULE('Windows API')
DPAPI_OpenPrinterA              PROCEDURE(*DP_LPSTR,*DP_HANDLE,*DP_PRINTER_DEFAULTS),BOOL,PASCAL,RAW,NAME('OpenPrinterA')
DPAPI_StartDocPrinterA          PROCEDURE(DP_HANDLE,DP_DWORD,*BYTE),DP_DWORD,PASCAL,NAME('StartDocPrinterA')
DPAPI_StartPagePrinter          PROCEDURE(DP_HANDLE),BOOL,PASCAL,NAME('StartPagePrinter')
DPAPI_WritePrinter              PROCEDURE(DP_HANDLE,DP_LPVOID,DP_DWORD,*DP_DWORD),BOOL,PASCAL,RAW,NAME('WritePrinter')
DPAPI_EndPagePrinter            PROCEDURE(DP_HANDLE),BOOL,PASCAL,PROC,NAME('EndPagePrinter')
DPAPI_EndDocPrinter             PROCEDURE(DP_HANDLE),BOOL,PASCAL,PROC,NAME('EndDocPrinter')
DPAPI_ClosePrinter              PROCEDURE(DP_HANDLE),BOOL,PASCAL,PROC,NAME('ClosePrinter')
            END ! MODULE
            
DP_FinalizeThreaded     PROCEDURE(STRING p:PrinterHandleStr)
DP_FinalizeUnThreaded   PROCEDURE(DP_HANDLE p:PrinterHandle)    
            END !MAP


                        

DirectPrintClass.Initialize           PROCEDURE(<STRING p:PrinterName>) !,VIRTUAL,BYTE           ! Declare Procedure
!   LOCAL DATA DECLARATIONS
!   ========================================================================================================================
hPrinter            DP_HANDLE
szPrinterName       DP_LPSTR(80)
dwJob               DP_DWORD
DocInfo             LIKE(DP_DOC_INFO_1)
DocDefaults         LIKE(DP_PRINTER_DEFAULTS)
lpDocInfo           BYTE,DIM(SIZE(DocDefaults)),OVER(DocDefaults)
LocDocName          DP_LPSTR(80)
LocDataType         DP_LPSTR(80)
LocOutputFile       DP_LPSTR(80)    

  CODE
!   MAIN LINE CODE
!   ========================================================================================================================
!   PROCEDURE   : Initialise
!   DESCRIPTION : Initialises the Driver and Settings for Writing TEXT ONLY to the Printer
!   ------------------------------------------------------------------------------------------------------------------------
!   PROTOTYPE   : DirectPrintClass.Initialise (<STRING p:PrinterName>), BYTE
!   PARAMETERS  : p:PrinterName - Name of the Printer to Connect to. If Ommitted, system will use the Default Printer
!   RETURNS     : TRUE if a Handle was obtained, False if otherwise (failed)
!   ========================================================================================================================
!
!   First, check that we are not already connect to the printer and return OK if we are.
!   ------------------------------------------------------------------------------------------------------------------------
    IF SELF.GetPrinterHandle() THEN RETURN(TRUE).

!   Create a Connection to the printer using OpenPrinterA which will return a handle to the printer
!   ------------------------------------------------------------------------------------------------------------------------
    IF SELF.PrinterInited
      return (TRUE)
    END

    IF NOT OMITTED(2) THEN
       szPrinterName   = p:PrinterName                          ! Move String Value to CString Type
    ELSE
       szPrinterName   = SELF.GetDefaultPrinter()
    END

    SELF.SetStructNull(DocDefaults,SIZE(DocDefaults))      ! Make sure this structure is all 0's
    IF NOT DPAPI_OpenPrinterA( szPrinterName, hPrinter, DocDefaults ) THEN
       RETURN(FALSE)
    END

!   Setup and Startup a Document for the Printer
!   ------------------------------------------------------------------------------------------------------------------------
    SELF.SetStructNull(DocInfo,SIZE(DocInfo))
    
    LocDocName = 'Direct Print'
    LocDataType = 'RAW'
    LocOutputFile = DP_NULL
    
    DocInfo.pDocName    = ADDRESS(LocDocName) ! 'Receipt Printer'
    DocInfo.pDataType   = ADDRESS(LocDataType) !'RAW'
    DocInfo.pOutputFile = ADDRESS(LocOutputFIle) ! NULL
    
    dwJob = DPAPI_StartDocPrinterA ( hPrinter, 1, lpDocInfo[1])

    IF NOT dwJob THEN
       DPAPI_ClosePrinter(hPrinter)
       RETURN (FALSE)
    END

!   Start a Page on the Printer
!   ------------------------------------------------------------------------------------------------------------------------
    IF NOT DPAPI_StartPagePrinter ( hPrinter ) THEN
       DPAPI_EndDocPrinter( hPrinter )
       DPAPI_ClosePrinter ( hPrinter )
       RETURN (FALSE)
    END
    SELF.SetPrinterHandle(hPrinter)    ! Save the handle for other modules to use
    SELF.PrinterInited=TRUE                  ! MGP 03/09/05 don't init printer twice
    RETURN(TRUE)
    

DirectPrintClass.Finalize             PROCEDURE(BYTE p:ThreadFlag = 0) !,VIRTUAL                   ! Declare Procedure    
!   DATA DECLARATIONS
LocPrnHandleStr STRING(10)

  CODE
!   MAIN LINE CODE
!   ========================================================================================================================
!   PROCEDURE   : Finalise
!   DESCRIPTION : Calls code based on parameter
!   ------------------------------------------------------------------------------------------------------------------------
!   PROTOTYPE   : DirectPrintClass.Finalize PROCEDURE(BYTE p:ThreadFlag = 0)
!   PARAMETERS  : 
!               :  BYTE p:ThreadFlag = 0  - Determines whether to START a thread for finalizing
!               :  Calling this procedure with a (1) parameter may increase responsiveness of system
!               :  as the Print Job is being closed and flushed to the printer.
!   RETURNS     : There are no return values
!   ========================================================================================================================
!
    IF SELF.PrinterInited
      IF p:ThreadFlag <> 0
        LocPrnHandleStr = SELF.GetPrinterHandle() 
        
        START(DP_FinalizeThreaded,10000,LocPrnHandleStr)
      ELSE
        DP_FinalizeUnThreaded(SELF.GetPrinterHandle())
      END !IF  
      SELF.SetPrinterHandle(0)                       ! Reset the Handle
      SELF.PrinterInited=FALSE                       ! MGP 050309 Maintain PrinterInited
    END
    RETURN
    
    
DirectPrintClass.GetDefaultPrinter    PROCEDURE()  !,STRING,PROTECTED                  ! Declare Procedure
! LOCAL DATA DECLARATIONS
PrinterName     STRING(80)
PrinterDriver   STRING(32)
PrinterPort     STRING(32)
PrinterDevice   STRING(132)
  CODE
!   MAIN LINE CODE
!   ========================================================================================================================
!   PROCEDURE   : GetDefaultPrinter
!   DESCRIPTION : Returns the Name of the Default Printer
!   ------------------------------------------------------------------------------------------------------------------------
!   PROTOTYPE   : DirectPrintClass.GetDefaultPrinter    PROCEDURE(),STRING
!   PARAMETERS  : None
!   RETURNS     : The Name of the Default Printer
!   ========================================================================================================================
!
  !  PrinterDevice   = GETINI('windows','device')
    PrinterName     =  PRINTER{DP_PROPPRINT:Device} !SELF.GetStringElement(1,',',PrinterDevice)
  !  PrinterDriver   = SELF.GetStringElement(2,',',PrinterDevice)
  !  PrinterPort     = SELF.GetStringElement(3,',',PrinterDevice)
    RETURN(CLIP(PrinterName))
    
    
DirectPrintClass.GetPrinterhandle     PROCEDURE()   !,DP_HANDLE,PROTECTED                   ! Declare Procedure    

  CODE
!   MAIN LINE CODE
!   ========================================================================================================================
!   PROCEDURE   : GetPrinterHandle
!   DESCRIPTION : Returns the Printer Handle in a Module Level Variable
!   ------------------------------------------------------------------------------------------------------------------------
!   PROTOTYPE   : DirectPrintClass.GetPrinterhandle     PROCEDURE(),DP_HANDLE
!   PARAMETERS  : None
!   RETURNS     : The value stored by SetPrinterHandle
!   ========================================================================================================================
!
    RETURN(SELF.PrinterHandle)
    
DirectPrintClass.GetStringElement     PROCEDURE(BYTE p:Element,STRING p:Seperator,STRING p:StringToSeperate)  ! ,STRING,PROTECTED ! Declare Procedure    

! LOCAL DATA
LOC:Position1        LONG
LOC:Position2        LONG
LOC:Element          STRING(40)
  CODE
!   MAIN LINE CODE
!   =======================================================================================================================
!   PROCEDURE   :   GetStringElement
!   DESCRIPTION :   Returns a Portion of a Element of a String
!   =======================================================================================================================
!
    LOC:Position1   = 1
    LOC:Position2   = 0
    LOOP p:Element TIMES
      LOC:Position2 = INSTRING(CLIP(p:Seperator),p:StringToSeperate,1,LOC:Position1)
      IF NOT LOC:Position2 THEN LOC:Position2 = LEN(p:StringToSeperate)+1.
      LOC:Element   = p:StringToSeperate[LOC:Position1 : LOC:Position2-1]
      LOC:Position1 = LOC:Position2 + 1
    END
    RETURN(LOC:Element)

    
DirectPrintClass.SetPrinterHandle     PROCEDURE(DP_HANDLE p:PrinterHandle)   ! ,PROTECTED    ! Declare Procedure    
  CODE
!   MAIN LINE CODE
!   ========================================================================================================================
!   PROCEDURE   : SetPrinterHandle
!   DESCRIPTION : Saves the Printer Handle in a Module Level Variable
!   ------------------------------------------------------------------------------------------------------------------------
!   PROTOTYPE   : DirectPrintClass.SetPrinterHandle     PROCEDURE(DP_HANDLE p:PrinterHandle)
!   PARAMETERS  : p:PrinterHandle   - The Handle Returned from the API Call to OpenPrinterA
!   RETURNS     : None
!   ========================================================================================================================
!
    SELF.PrinterHandle   = p:PrinterHandle
    RETURN
    
    
DirectPrintClass.SetStructNull        PROCEDURE(*STRING p:Structure, LONG p:Size)  !,PROTECTED   ! Declare Procedure    

! LOCAL DATA
lLoopPointer    LONG(0)
  CODE
!   MAIN LINE CODE
!   ========================================================================================================================
!   PROCEDURE   : SetStructNull
!   DESCRIPTION : Ensures that a Structure is filled with all NULLS as required by some API Calls
!   ------------------------------------------------------------------------------------------------------------------------
!   PROTOTYPE   : DirectPrintClass.SetStructNull ( *STRING p:Structure, LONG p:Size)
!   PARAMETERS  : p:Structure   - A Pointer to a Structure of STRING or GROUP
!                 p:Size        - The Size in Bytes of the Structure to set
!   RETURNS     : none
!   ========================================================================================================================
!
    LOOP lLoopPointer = 1 TO p:Size
      p:Structure[lLoopPointer] = '<00>'
    END
    RETURN

    
DirectPrintClass.WriteText            PROCEDURE(STRING p:TextToWrite,ULONG p:Len,BYTE p:AppendCRLF=TRUE)   ! ,VIRTUAL,BYTE,PROC ! Declare Procedure    

!   DATA DECLARATIONS
!   ========================================================================================================================
szBuffer        DP_LPSTR(64000)
lpBuffer        DP_LPVOID
dwCount         DP_DWORD
dwBytesWritten  DP_DWORD
  CODE
!   MAIN LINE CODE
!   ========================================================================================================================
!   PROCEDURE   : WriteText
!   DESCRIPTION : Outputs Text to the Printer as Specified in the Parameters
!   ------------------------------------------------------------------------------------------------------------------------
!   PROTOTYPE   : DirectPrintClass.WriteText            PROCEDURE(STRING p:TextToWrite,ULONG p:Len,BYTE p:AppendCRLF=TRUE),BYTE,PROC ! Declare Procedure    
!   PARAMETERS  : p:TextToWrite     - A String of Text to write to the Printer
!   RETURNS     : TRUE if Writing was Sucessful, False otherwise
!   ========================================================================================================================
!
!   Check that the printer has been initialised. Cause if it hasn't then we cannot write to it
!   -----------------------------------------------------------------------------------------------------------------------
    IF NOT SELF.GetPrinterHandle() THEN RETURN(FALSE).

!   IF AppendCRLF is set true or is Omitted (Default True) then append a CRLF to the end of the line to print
!   -----------------------------------------------------------------------------------------------------------------------
    szBuffer        = p:TextToWrite
    dwCount         = p:Len
    IF p:AppendCRLF THEN
      szBuffer = CLIP(szBuffer) & '<13,10>'
      dwCount=dwCount+2
    END

!   Set up the parameters for the write to the printer
!   -----------------------------------------------------------------------------------------------------------------------
    lpBuffer        = ADDRESS(szBuffer)
    dwBytesWritten  = 0

    IF NOT DPAPI_WritePrinter (SELF.GetPrinterHandle(), lpBuffer, dwCount, dwBytesWritten) THEN
       RETURN(FALSE)
    END
    RETURN(TRUE)

DirectPrintClass.LinePrint            PROCEDURE(STRING p:TextToWrite)   ! ,VIRTUAL ! Declare Procedure

!   DATA DECLARATIONS
!   ========================================================================================================================

  CODE
!   MAIN LINE CODE
!   ========================================================================================================================
!   PROCEDURE   : LinePrint
!   DESCRIPTION : Wrapper to WriteText
!   ------------------------------------------------------------------------------------------------------------------------
!   PROTOTYPE   : DirectPrintClass.LinePrint            PROCEDURE(STRING p:TextToWrite),PROC ! Declare Procedure
!   PARAMETERS  : p:TextToWrite     - A String of Text to write to the Printer
!   ========================================================================================================================
!
    if SELF.WriteText(clip(p:TextToWrite), len(clip(p:TextToWrite))) THEN .  !appends CRLF

    
DirectPrintClass.PrintJobOpen         PROCEDURE()   ! ,BYTE,PROTECTED                         ! Declare Procedure    
  CODE
 ! Open the Windows Print Job
! IF PrtMode = 'Windows'
   RETURN (CLIP(SELF.Initialize(PRINTER{DP_PROPPRINT:Device})))
! END
! RETURN (TRUE)


DirectPrintClass.PrintJobClose        PROCEDURE()  ! ,PROTECTED                         ! Declare Procedure

  CODE
 ! Close Windows Print Job
! IF PrtMode = 'Windows'
   SELF.Finalize()
! END


DP_FinalizeThreaded     PROCEDURE(STRING p:PrinterHandleStr)

PrnHandle       LONG
!   MAIN LINE CODE
!   ========================================================================================================================
!   PROCEDURE   : DP_FinalizeThreaded
!   DESCRIPTION : Used Primarily as a STARTed procedure. Calls DP_FinalizeUnThreaded immediately.
!               : STARTing this procedure will allow the Printer Job to finish as a seperate
!               : thread of processing while main body of code continues on.
!   ------------------------------------------------------------------------------------------------------------------------
!   PROTOTYPE   : DP_FinalizeThreaded   PROCEDURE(STRING p:PrinterHandleStr)    
!   PARAMETERS  : 
!               : - p:PrinterHandleStr - Handle to the Open Printer.as a String.
!   RETURNS     : There are no return values
!   ========================================================================================================================
!


        CODE
        PrnHandle = p:PrinterHandleStr  ! Convert digits of STRING into LONG
        DP_FinalizeUnThreaded(PrnHandle)
        

DP_FinalizeUnThreaded   PROCEDURE(DP_HANDLE p:PrinterHandle)    


!   DATA DECLARATIONS
szBuffer        DP_LPSTR(64000)
lpBuffer        DP_LPVOID
dwCount         DP_DWORD
dwBytesWritten  DP_DWORD
  CODE
!   MAIN LINE CODE
!   ========================================================================================================================
!   PROCEDURE   : DP_FinalizeUnThreaded
!   DESCRIPTION : Closes the Print Job and Releases the Printer Handle
!   ------------------------------------------------------------------------------------------------------------------------
!   PROTOTYPE   : DP_FinalizeUnThreaded   PROCEDURE(DP_HANDLE p:PrinterHandle)    
!   PARAMETERS  : 
!               : - p:PrinterHandle Handle to the Open Printer.
!   RETURNS     : There are no return values
!   ========================================================================================================================
!
      DPAPI_EndPagePrinter ( p:PrinterHandle )     ! Tell the Spooler to End the Page
      DPAPI_EndDocPrinter  ( p:PrinterHandle )     ! Tell the Spooler to End the Document
      DPAPI_ClosePrinter   ( p:PrinterHandle )     ! Tell the Printer the Job is Finished
      RETURN
